local info, lib = ...

local _DB = {}
lib._DB = _DB

local function resolveDefaults( db, k )
	local _def = rawget( db, "_defaults" )
	if _def and _def[k] then
		return _def[k]
	end

	-- TODO: Add inheritance
end

function _DB:__index( k )
	if _DB[k] ~= nil then
		return _DB[k]
	end

	local _table = rawget( self, "_table" )
	if _table and _table[self._profile][k] ~= nil then
		return _table[self._profile][k]
	end

	return resolveDefaults( self, k )
end

function _DB:__newindex( k, v )
	self._table[self._profile][k] = v
end

function _DB:SetProfile( prof )
	rawset( self, "_profile", prof )

	if self._table and not self._table[prof] then
		self._table[prof] = {}
	end
end

local function copyto( from, to, seen )
	seen = seen or {}
	if seen[from] then return seen[from] end

	for k, v in pairs( from ) do
		if type( v ) == "table" then
			to[k] = copyto( v, {}, seen )
		else
			to[k] = v
		end
	end

	seen[from] = to
	return to
end

function _DB:ImportProfile( prof )
	if not self._table then return end

	local from = self._table[prof]
	if not from then return end

	copyto( from, self )
end

function _DB:GetProfiles()
	local ret = {}
	for k, v in pairs( self:GetTable() ) do
		if k ~= "keys" then table.insert( ret, k ) end
	end
	return ret
end

function _DB:GetProfile()
	return self._profile
end

function _DB:SetTable( tbl )
	rawset( self, "_table", tbl )
end

function _DB:GetTable()
	return self._table
end

function _DB:SetDefaults( def )
	rawset( self, "_defaults", def )
end

function _DB:GetDefaults()
	return self._defaults
end

function _DB:InitTable()
	local tbl = self:GetTable()

	tbl.keys = tbl.keys or {}

	self:SetProfile( tbl.keys[lib.GetCharProfile()] or lib.GetCharProfile() )
end

function _DB:Wipe( all )
	if all then
		self:SetTable( {} )
		self:InitTable()
	else
		self:GetTable()[self:GetProfile()] = {}
	end
end
